package com.hungraim.ltc.system.service.impl;

import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.hungraim.ltc.constant.AuthConstants;
import com.hungraim.ltc.pojo.entity.system.SystemInterface;
import com.hungraim.ltc.system.dao.SystemInterfaceMapper;
import com.hungraim.ltc.system.service.ISystemInterfaceService;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.*;

/**
 * @author mq
 */
@Service
@Slf4j
public class SystemInterfaceServiceImpl extends ServiceImpl<SystemInterfaceMapper, SystemInterface> implements ISystemInterfaceService {

    private final SystemInterfaceMapper systemInterfaceMapper;


    private final SystemResourceServiceImpl systemResourceService;
    private final RedisTemplate redisTemplate;

    @Autowired
    public SystemInterfaceServiceImpl(SystemInterfaceMapper systemInterfaceMapper, SystemResourceServiceImpl systemResourceService, RedisTemplate redisTemplate) {
        this.systemInterfaceMapper = systemInterfaceMapper;
        this.systemResourceService = systemResourceService;
        this.redisTemplate = redisTemplate;
    }

    @Override
    public void listResourceInterface() {
        try {
            redisTemplate.delete(AuthConstants.PERMISSION_RESOURCE_INTERFACE_KEY);
            redisTemplate.delete(AuthConstants.PERMISSION_ROLES_RESOURCE_KEY);
            Map<String, List<Long>> roleResourcesMap = new TreeMap<>();
            Map<String, List<SystemInterface>> stringStringHashMap = new HashMap<>();
            List<Long> roles = systemInterfaceMapper.listRole();
            String roleId;
            for (Long  role : roles) {
                List<Long> roleResources = systemInterfaceMapper.listRoleResource(role);
                //保存资源和接口
                for (Long roleResource:roleResources){
                    List<SystemInterface> systemInterfaces=systemInterfaceMapper.listInterface(roleResource);
                    stringStringHashMap.put(roleResource.toString(),systemInterfaces);
                }
                // 转换 roles -> ROLE_{roleId}
                roleId=AuthConstants.AUTHORITY_PREFIX + role;
                //保存角色和资源
                roleResourcesMap.put(roleId,roleResources);
            }
            redisTemplate.opsForHash().putAll(AuthConstants.PERMISSION_ROLES_RESOURCE_KEY, roleResourcesMap);
            redisTemplate.opsForHash().putAll(AuthConstants.PERMISSION_RESOURCE_INTERFACE_KEY, stringStringHashMap);
        }catch (RuntimeException e){
            e.printStackTrace();
        }
    }
}
